@@ -30,7 +30,7 @@ import re
 import time
 from itertools import groupby
 
-from gi.repository import GdkPixbuf, GObject, Gtk
+from gi.repository import GdkPixbuf, GLib, GObject, Gtk
 
 import gpodder
 from gpodder import coverart, model, query, util
@@ -64,8 +64,8 @@ class GEpisode(model.PodcastEpisode):
             length_str = '%s; ' % util.format_filesize(self.file_size)
         else:
             length_str = ''
-        return ('<b>%s</b>\n<small>%s' + _('released %s') +
-                '; ' + _('from %s') + '</small>') % (
+        return ('<b>%s</b>\n<small>%s' + _('released %s')
+                + '; ' + _('from %s') + '</small>') % (
                 html.escape(re.sub(r'\s+', ' ', self.title)),
                 html.escape(length_str),
                 html.escape(self.pubdate_prop),
@@ -82,8 +82,8 @@ class GEpisode(model.PodcastEpisode):
         downloaded_string = self.get_age_string()
         if not downloaded_string:
             downloaded_string = _('today')
-        return ('<b>%s</b>\n<small>%s; %s; ' + _('downloaded %s') +
-                '; ' + _('from %s') + '</small>') % (
+        return ('<b>%s</b>\n<small>%s; %s; ' + _('downloaded %s')
+                + '; ' + _('from %s') + '</small>') % (
                 html.escape(self.title),
                 html.escape(util.format_filesize(self.file_size)),
                 html.escape(played_string),
@@ -113,38 +113,39 @@ class SeparatorMarker(object): pass
 
 
 class BackgroundUpdate(object):
-    def __init__(self, model, episodes, include_description):
+    def __init__(self, model, episodes):
         self.model = model
         self.episodes = episodes
-        self.include_description = include_description
         self.index = 0
 
     def update(self):
         model = self.model
-        include_description = self.include_description
 
         started = time.time()
         while self.episodes:
             episode = self.episodes.pop(0)
             base_fields = (
-                (model.C_URL, episode.url),
-                (model.C_TITLE, episode.title),
-                (model.C_EPISODE, episode),
-                (model.C_PUBLISHED_TEXT, episode.cute_pubdate()),
-                (model.C_PUBLISHED, episode.published),
+                model.C_URL, episode.url,
+                model.C_TITLE, episode.title,
+                model.C_EPISODE, episode,
+                model.C_PUBLISHED_TEXT, episode.cute_pubdate(),
+                model.C_PUBLISHED, episode.published,
             )
-            update_fields = model.get_update_fields(episode, include_description)
+            update_fields = model.get_update_fields(episode)
             try:
                 it = model.get_iter((self.index,))
             # fix #727 the tree might be invalid when trying to update so discard the exception
             except ValueError:
                 break
-            model.set(it, *(x for fields in (base_fields, update_fields)
-                            for pair in fields for x in pair))
+            # model.get_update_fields() takes 38-67% of each iteration, depending on episode status
+            # with downloaded episodes using the most time
+            # model.set(), excluding the field expansion, takes 33-62% of each iteration
+            # and each iteration takes 1ms or more on slow machines
+            model.set(it, *(base_fields + update_fields))
             self.index += 1
 
-            # Check for the time limit of 20 ms after each 50 rows processed
-            if self.index % 50 == 0 and (time.time() - started) > 0.02:
+            # Check for the time limit of 500ms after each 50 rows processed
+            if self.index % 50 == 0 and (time.time() - started) > 0.5:
                 break
 
         return bool(self.episodes)
@@ -169,14 +170,12 @@ class EpisodeListModel(Gtk.ListStore):
     # Steps for the "downloading" icon progress
     PROGRESS_STEPS = 20
 
-    def __init__(self, config, on_filter_changed=lambda has_episodes: None):
+    def __init__(self, on_filter_changed=lambda has_episodes: None):
         Gtk.ListStore.__init__(self, str, str, str, object, str, str, str,
                                str, bool, bool, bool, GObject.TYPE_INT64,
                                GObject.TYPE_INT64, str, bool,
                                GObject.TYPE_INT64, bool, str, GObject.TYPE_INT64, str, GObject.TYPE_INT64)
 
-        self._config = config
-
         # Callback for when the filter / list changes, gets one parameter
         # (has_episodes) that is True if the list has any episodes
         self._on_filter_changed = on_filter_changed
@@ -192,11 +191,13 @@ class EpisodeListModel(Gtk.ListStore):
         # Are we currently showing "all episodes"/section or a single channel?
         self._section_view = False
 
+        self.icon_theme = Gtk.IconTheme.get_default()
+        self.ICON_WEB_BROWSER = 'web-browser'
         self.ICON_AUDIO_FILE = 'audio-x-generic'
         self.ICON_VIDEO_FILE = 'video-x-generic'
         self.ICON_IMAGE_FILE = 'image-x-generic'
         self.ICON_GENERIC_FILE = 'text-x-generic'
-        self.ICON_DOWNLOADING = Gtk.STOCK_GO_DOWN
+        self.ICON_DOWNLOADING = 'go-down'
         self.ICON_DELETED = 'edit-delete'
         self.ICON_ERROR = 'dialog-error'
 
@@ -209,6 +210,17 @@ class EpisodeListModel(Gtk.ListStore):
             #     http://gpodder.org/bug/553
             self.ICON_DELETED = 'archive-remove'
 
+        # Caching config values is faster than accessing them directly from config.ui.gtk.episode_list.*
+        # and is easier to maintain then threading them through every method call.
+        self._config_ui_gtk_episode_list_always_show_new = False
+        self._config_ui_gtk_episode_list_trim_title_prefix = False
+        self._config_ui_gtk_episode_list_descriptions = False
+
+    def cache_config(self, config):
+        self._config_ui_gtk_episode_list_always_show_new = config.ui.gtk.episode_list.always_show_new
+        self._config_ui_gtk_episode_list_trim_title_prefix = config.ui.gtk.episode_list.trim_title_prefix
+        self._config_ui_gtk_episode_list_descriptions = config.ui.gtk.episode_list.descriptions
+
     def _format_filesize(self, episode):
         if episode.file_size > 0:
             return util.format_filesize(episode.file_size, digits=1)
@@ -279,32 +291,33 @@ class EpisodeListModel(Gtk.ListStore):
     def get_search_term(self):
         return self._search_term
 
-    def _format_description(self, episode, include_description=False):
-        title = episode.trimmed_title
+    def _format_description(self, episode):
+        d = []
 
+        title = episode.trimmed_title if self._config_ui_gtk_episode_list_trim_title_prefix else episode.title
         if episode.state != gpodder.STATE_DELETED and episode.is_new:
-            yield '<b>'
-            yield html.escape(title)
-            yield '</b>'
+            d.append('<b>')
+            d.append(html.escape(title))
+            d.append('</b>')
         else:
-            yield html.escape(title)
+            d.append(html.escape(title))
 
-        if include_description:
-            yield '\n'
+        if self._config_ui_gtk_episode_list_descriptions:
+            d.append('\n')
             if self._section_view:
-                yield _('from %s') % html.escape(episode.channel.title)
+                d.append(_('from %s') % html.escape(episode.channel.title))
             else:
                 description = episode.one_line_description()
                 if description.startswith(title):
                     description = description[len(title):].strip()
-                yield html.escape(description)
+                d.append(html.escape(description))
 
-    def replace_from_channel(self, channel, include_description=False):
+        return ''.join(d)
+
+    def replace_from_channel(self, channel):
         """
         Add episode from the given channel to this model.
         Downloading should be a callback.
-        include_description should be a boolean value (True if description
-        is to be added to the episode row, or False if not)
         """
 
         # Remove old episodes in the list store
@@ -324,14 +337,14 @@ class EpisodeListModel(Gtk.ListStore):
         for _ in range(len(episodes)):
             self.append()
 
-        self._update_from_episodes(episodes, include_description)
+        self._update_from_episodes(episodes)
 
-    def _update_from_episodes(self, episodes, include_description):
+    def _update_from_episodes(self, episodes):
         if self.background_update_tag is not None:
-            GObject.source_remove(self.background_update_tag)
+            GLib.source_remove(self.background_update_tag)
 
-        self.background_update = BackgroundUpdate(self, episodes, include_description)
-        self.background_update_tag = GObject.idle_add(self._update_background)
+        self.background_update = BackgroundUpdate(self, episodes)
+        self.background_update_tag = GLib.idle_add(self._update_background)
 
     def _update_background(self):
         if self.background_update is not None:
@@ -344,7 +357,7 @@ class EpisodeListModel(Gtk.ListStore):
 
         return False
 
-    def update_all(self, include_description=False):
+    def update_all(self):
         if self.background_update is None:
             episodes = [row[self.C_EPISODE] for row in self]
         else:
@@ -353,37 +366,37 @@ class EpisodeListModel(Gtk.ListStore):
             # ...and also include episodes that still need to be initialized
             episodes.extend(self.background_update.episodes)
 
-        self._update_from_episodes(episodes, include_description)
+        self._update_from_episodes(episodes)
 
-    def update_by_urls(self, urls, include_description=False):
+    def update_by_urls(self, urls):
         for row in self:
             if row[self.C_URL] in urls:
-                self.update_by_iter(row.iter, include_description)
+                self.update_by_iter(row.iter)
 
-    def update_by_filter_iter(self, iter, include_description=False):
+    def update_by_filter_iter(self, iter):
         # Convenience function for use by "outside" methods that use iters
         # from the filtered episode list model (i.e. all UI things normally)
         iter = self._sorter.convert_iter_to_child_iter(iter)
-        self.update_by_iter(self._filter.convert_iter_to_child_iter(iter),
-                include_description)
+        self.update_by_iter(self._filter.convert_iter_to_child_iter(iter))
 
-    def get_update_fields(self, episode, include_description):
-        show_bullet = False
-        show_padlock = False
-        show_missing = False
-        status_icon = None
+    def get_update_fields(self, episode):
         tooltip = []
+        status_icon = None
         view_show_undeleted = True
         view_show_downloaded = False
         view_show_unplayed = False
-        icon_theme = Gtk.IconTheme.get_default()
 
         if episode.downloading:
-            tooltip.append('%s %d%%' % (_('Downloading'),
-                int(episode.download_task.progress * 100)))
-
-            index = int(self.PROGRESS_STEPS * episode.download_task.progress)
-            status_icon = 'gpodder-progress-%d' % index
+            task = episode.download_task
+            if task.status in (task.PAUSING, task.PAUSED):
+                tooltip.append('%s %d%%' % (_('Paused'),
+                    int(task.progress * 100)))
+                status_icon = 'media-playback-pause'
+            else:
+                tooltip.append('%s %d%%' % (_('Downloading'),
+                    int(task.progress * 100)))
+                index = int(self.PROGRESS_STEPS * task.progress)
+                status_icon = 'gpodder-progress-%d' % index
 
             view_show_downloaded = True
             view_show_unplayed = True
@@ -393,13 +406,8 @@ class EpisodeListModel(Gtk.ListStore):
                 status_icon = self.ICON_DELETED
                 view_show_undeleted = False
             elif episode.state == gpodder.STATE_DOWNLOADED:
-                tooltip = []
                 view_show_downloaded = True
                 view_show_unplayed = episode.is_new
-                show_bullet = episode.is_new
-                show_padlock = episode.archive
-                show_missing = not episode.file_exists()
-                filename = episode.local_filename(create=False, check_only=True)
 
                 file_type = episode.file_type()
                 if file_type == 'audio':
@@ -415,50 +423,44 @@ class EpisodeListModel(Gtk.ListStore):
                     tooltip.append(_('Downloaded file'))
                     status_icon = self.ICON_GENERIC_FILE
 
-                # Try to find a themed icon for this file
-                # doesn't work on win32 (opus files are showed as text)
-                if filename is not None and have_gio and not gpodder.ui.win32:
-                    file = Gio.File.new_for_path(filename)
-                    if file.query_exists():
-                        file_info = file.query_info('*', Gio.FileQueryInfoFlags.NONE, None)
-                        icon = file_info.get_icon()
-                        for icon_name in icon.get_names():
-                            if icon_theme.has_icon(icon_name):
-                                status_icon = icon_name
-                                break
-
-                if show_missing:
+                if not episode.file_exists():
                     tooltip.append(_('missing file'))
                 else:
-                    if show_bullet:
-                        if file_type == 'image':
-                            tooltip.append(_('never displayed'))
-                        elif file_type in ('audio', 'video'):
+                    if episode.is_new:
+                        if file_type in ('audio', 'video'):
                             tooltip.append(_('never played'))
+                        elif file_type == 'image':
+                            tooltip.append(_('never displayed'))
                         else:
                             tooltip.append(_('never opened'))
                     else:
-                        if file_type == 'image':
-                            tooltip.append(_('displayed'))
-                        elif file_type in ('audio', 'video'):
+                        if file_type in ('audio', 'video'):
                             tooltip.append(_('played'))
+                        elif file_type == 'image':
+                            tooltip.append(_('displayed'))
                         else:
                             tooltip.append(_('opened'))
-                    if show_padlock:
+                    if episode.archive:
                         tooltip.append(_('deletion prevented'))
 
                 if episode.total_time > 0 and episode.current_position:
-                    tooltip.append('%d%%' % (100. * float(episode.current_position) /
-                                             float(episode.total_time),))
+                    tooltip.append('%d%%' % (
+                        100. * float(episode.current_position) / float(episode.total_time)))
             elif episode._download_error is not None:
                 tooltip.append(_('ERROR: %s') % episode._download_error)
                 status_icon = self.ICON_ERROR
                 if episode.state == gpodder.STATE_NORMAL and episode.is_new:
-                    view_show_downloaded = self._config.ui.gtk.episode_list.always_show_new
+                    view_show_downloaded = self._config_ui_gtk_episode_list_always_show_new
+                    view_show_unplayed = True
+            elif not episode.url:
+                tooltip.append(_('No downloadable content'))
+                status_icon = self.ICON_WEB_BROWSER
+                if episode.state == gpodder.STATE_NORMAL and episode.is_new:
+                    view_show_downloaded = self._config_ui_gtk_episode_list_always_show_new
                     view_show_unplayed = True
             elif episode.state == gpodder.STATE_NORMAL and episode.is_new:
                 tooltip.append(_('New episode'))
-                view_show_downloaded = self._config.ui.gtk.episode_list.always_show_new
+                view_show_downloaded = self._config_ui_gtk_episode_list_always_show_new
                 view_show_unplayed = True
 
         if episode.total_time:
@@ -468,33 +470,34 @@ class EpisodeListModel(Gtk.ListStore):
 
         tooltip = ', '.join(tooltip)
 
-        description = ''.join(self._format_description(episode, include_description))
-        return (
-                (self.C_STATUS_ICON, status_icon),
-                (self.C_VIEW_SHOW_UNDELETED, view_show_undeleted),
-                (self.C_VIEW_SHOW_DOWNLOADED, view_show_downloaded),
-                (self.C_VIEW_SHOW_UNPLAYED, view_show_unplayed),
-                (self.C_DESCRIPTION, description),
-                (self.C_TOOLTIP, tooltip),
-                (self.C_TIME, episode.get_play_info_string()),
-                (self.C_TIME_VISIBLE, bool(episode.total_time)),
-                (self.C_TOTAL_TIME, episode.total_time),
-                (self.C_LOCKED, episode.archive),
-                (self.C_FILESIZE_TEXT, self._format_filesize(episode)),
-                (self.C_FILESIZE, episode.file_size),
+        description = self._format_description(episode)
+        time = episode.get_play_info_string()
+        filesize = self._format_filesize(episode)
 
-                (self.C_TIME_AND_SIZE, "%s\n<small>%s</small>"
-                    % (episode.get_play_info_string(), self._format_filesize(episode) if episode.file_size > 0 else "")),
-                (self.C_TOTAL_TIME_AND_SIZE, episode.total_time),
-                (self.C_FILESIZE_AND_TIME_TEXT, "%s\n<small>%s</small>"
-                    % (self._format_filesize(episode) if episode.file_size > 0 else "", episode.get_play_info_string())),
-                (self.C_FILESIZE_AND_TIME, episode.file_size),
+        return (
+                self.C_STATUS_ICON, status_icon,
+                self.C_VIEW_SHOW_UNDELETED, view_show_undeleted,
+                self.C_VIEW_SHOW_DOWNLOADED, view_show_downloaded,
+                self.C_VIEW_SHOW_UNPLAYED, view_show_unplayed,
+                self.C_DESCRIPTION, description,
+                self.C_TOOLTIP, tooltip,
+                self.C_TIME, time,
+                self.C_TIME_VISIBLE, bool(episode.total_time),
+                self.C_TOTAL_TIME, episode.total_time,
+                self.C_LOCKED, episode.archive,
+                self.C_FILESIZE_TEXT, filesize,
+                self.C_FILESIZE, episode.file_size,
+
+                self.C_TIME_AND_SIZE, "%s\n<small>%s</small>" % (time, filesize if episode.file_size > 0 else ""),
+                self.C_TOTAL_TIME_AND_SIZE, episode.total_time,
+                self.C_FILESIZE_AND_TIME_TEXT, "%s\n<small>%s</small>" % (filesize if episode.file_size > 0 else "", time),
+                self.C_FILESIZE_AND_TIME, episode.file_size,
         )
 
-    def update_by_iter(self, iter, include_description=False):
+    def update_by_iter(self, iter):
         episode = self.get_value(iter, self.C_EPISODE)
         if episode is not None:
-            self.set(iter, *(x for pair in self.get_update_fields(episode, include_description) for x in pair))
+            self.set(iter, *self.get_update_fields(episode))
 
 
 class PodcastChannelProxy:
@@ -585,8 +588,10 @@ class PodcastListModel(Gtk.ListStore):
 
         self._cover_cache = {}
         self._max_image_side = 40
+        self._scale = 1
         self._cover_downloader = cover_downloader
 
+        self.icon_theme = Gtk.IconTheme.get_default()
         self.ICON_DISABLED = 'media-playback-pause'
         self.ICON_ERROR = 'dialog-warning'
 
@@ -653,8 +658,9 @@ class PodcastListModel(Gtk.ListStore):
     def get_search_term(self):
         return self._search_term
 
-    def set_max_image_size(self, size):
-        self._max_image_side = size
+    def set_max_image_size(self, size, scale):
+        self._max_image_side = size * scale
+        self._scale = scale
         self._cover_cache = {}
 
     def _resize_pixbuf_keep_ratio(self, url, pixbuf):
@@ -663,31 +669,27 @@ class PodcastListModel(Gtk.ListStore):
         Returns None if the pixbuf does not need to be
         resized or the newly resized pixbuf if it does.
         """
-        changed = False
-        result = None
-
         if url in self._cover_cache:
             return self._cover_cache[url]
 
-        # Resize if too wide
-        if pixbuf.get_width() > self._max_image_side:
-            f = float(self._max_image_side) / pixbuf.get_width()
-            (width, height) = (int(pixbuf.get_width() * f), int(pixbuf.get_height() * f))
-            pixbuf = pixbuf.scale_simple(width, height, GdkPixbuf.InterpType.BILINEAR)
-            changed = True
+        max_side = self._max_image_side
+        w_cur = pixbuf.get_width()
+        h_cur = pixbuf.get_height()
 
-        # Resize if too high
-        if pixbuf.get_height() > self._max_image_side:
-            f = float(self._max_image_side) / pixbuf.get_height()
-            (width, height) = (int(pixbuf.get_width() * f), int(pixbuf.get_height() * f))
-            pixbuf = pixbuf.scale_simple(width, height, GdkPixbuf.InterpType.BILINEAR)
-            changed = True
+        if w_cur <= max_side and h_cur <= max_side:
+            return None
 
-        if changed:
-            self._cover_cache[url] = pixbuf
-            result = pixbuf
+        f = max_side / (w_cur if w_cur >= h_cur else h_cur)
+        w_new = int(w_cur * f)
+        h_new = int(h_cur * f)
 
-        return result
+        logger.debug("Scaling cover image: url=%s from %ix%i to %ix%i",
+                     url, w_cur, h_cur, w_new, h_new)
+        pixbuf = pixbuf.scale_simple(w_new, h_new,
+                                     GdkPixbuf.InterpType.BILINEAR)
+
+        self._cover_cache[url] = pixbuf
+        return pixbuf
 
     def _resize_pixbuf(self, url, pixbuf):
         if pixbuf is None:
@@ -697,14 +699,13 @@ class PodcastListModel(Gtk.ListStore):
 
     def _overlay_pixbuf(self, pixbuf, icon):
         try:
-            icon_theme = Gtk.IconTheme.get_default()
-            emblem = icon_theme.load_icon(icon, self._max_image_side / 2, 0)
+            emblem = self.icon_theme.load_icon(icon, self._max_image_side / 2, 0)
             (width, height) = (emblem.get_width(), emblem.get_height())
             xpos = pixbuf.get_width() - width
             ypos = pixbuf.get_height() - height
             if ypos < 0:
                 # need to resize overlay for none standard icon size
-                emblem = icon_theme.load_icon(icon, pixbuf.get_height() - 1, 0)
+                emblem = self.icon_theme.load_icon(icon, pixbuf.get_height() - 1, 0)
                 (width, height) = (emblem.get_width(), emblem.get_height())
                 xpos = pixbuf.get_width() - width
                 ypos = pixbuf.get_height() - height
@@ -726,8 +727,9 @@ class PodcastListModel(Gtk.ListStore):
             if self._max_image_side not in (pixbuf.get_width(), pixbuf.get_height()):
                 logger.debug("cached thumb wrong size: %r != %i", (pixbuf.get_width(), pixbuf.get_height()), self._max_image_side)
                 return None
+            return pixbuf
         except Exception as e:
-            logger.warn('Could not load cached cover art for %s', channel.url, exc_info=True)
+            logger.warning('Could not load cached cover art for %s', channel.url, exc_info=True)
             channel.cover_thumb = None
             channel.save()
             return None
@@ -758,6 +760,8 @@ class PodcastListModel(Gtk.ListStore):
         if pixbuf_overlay is None:
             # load cover if it's not in cache
             pixbuf = self._cover_downloader.get_cover(channel, avoid_downloading=True)
+            if pixbuf is None:
+                return None
             pixbuf_overlay = self._resize_pixbuf(channel.url, pixbuf)
             self._save_cached_thumb(channel, pixbuf_overlay)
 
@@ -772,7 +776,10 @@ class PodcastListModel(Gtk.ListStore):
 
     def _get_pill_image(self, channel, count_downloaded, count_unplayed):
         if count_unplayed > 0 or count_downloaded > 0:
-            return draw.draw_pill_pixbuf('{:n}'.format(count_unplayed), '{:n}'.format(count_downloaded), widget=self.widget)
+            return draw.draw_pill_pixbuf('{:n}'.format(count_unplayed),
+                                         '{:n}'.format(count_downloaded),
+                                         widget=self.widget,
+                                         scale=self._scale)
         else:
             return None
 
@@ -835,13 +842,13 @@ class PodcastListModel(Gtk.ListStore):
                     # C_DOWNLOADS, C_COVER_VISIBLE, C_SECTION
                     0, False, section.title)
 
-        if config.podcast_list_view_all and channels:
+        if config.ui.gtk.podcast_list.all_episodes and channels:
             all_episodes = PodcastChannelProxy(db, config, channels, '', self)
             iter = self.append(channel_to_row(all_episodes))
             self.update_by_iter(iter)
 
             # Separator item
-            if not config.podcast_list_sections:
+            if not config.ui.gtk.podcast_list.sections:
                 self.append(('', '', '', None, SeparatorMarker, None, '',
                     True, True, True, True, True, True, 0, False, ''))
 
@@ -851,13 +858,13 @@ class PodcastListModel(Gtk.ListStore):
         def key_func(channel):
             return (channel.group_by, model.Model.podcast_sort_key(channel))
 
-        if config.podcast_list_sections:
+        if config.ui.gtk.podcast_list.sections:
             groups = groupby(sorted(channels, key=key_func), groupby_func)
         else:
             groups = [(None, sorted(channels, key=model.Model.podcast_sort_key))]
 
         for section, section_channels in groups:
-            if config.podcast_list_sections and section is not None:
+            if config.ui.gtk.podcast_list.sections and section is not None:
                 section_channels = list(section_channels)
                 section_obj = PodcastChannelProxy(db, config, section_channels, section, self)
                 iter = self.append(section_to_row(section_obj))
