@@ -26,11 +26,10 @@ import logging
 import re
 import urllib
 import xml.etree.ElementTree
+from functools import lru_cache
 from html.parser import HTMLParser
 from urllib.parse import parse_qs
 
-import requests
-
 import gpodder
 from gpodder import registry, util
 
@@ -149,8 +148,21 @@ hls_formats = [
 ]
 hls_formats_dict = dict(hls_formats)
 
-V3_API_ENDPOINT = 'https://www.googleapis.com/youtube/v3'
 CHANNEL_VIDEOS_XML = 'https://www.youtube.com/feeds/videos.xml'
+WATCH_ENDPOINT = 'https://www.youtube.com/watch?bpctr=9999999999&has_verified=1&v='
+
+# The page may contain "};" sequences inside the initial player response.
+# Use a greedy match with script end tag, and fallback to a non-greedy match without.
+INITIAL_PLAYER_RESPONSE_RE1 = r'ytInitialPlayerResponse\s*=\s*({.+})\s*;\s*</script'
+INITIAL_PLAYER_RESPONSE_RE2 = r'ytInitialPlayerResponse\s*=\s*({.+?})\s*;'
+
+
+def get_ipr(page):
+    for regex in (INITIAL_PLAYER_RESPONSE_RE1, INITIAL_PLAYER_RESPONSE_RE2):
+        ipr = re.search(regex, page)
+        if ipr is not None:
+            return ipr
+    return None
 
 
 class YouTubeError(Exception):
@@ -189,6 +201,67 @@ def youtube_real_download_url(config, episode, allow_partial):
     return None if res == episode.url else res
 
 
+def youtube_get_old_endpoint(vid):
+    # TODO: changing 'detailpage' to 'embedded' allows age-restricted content
+    url = 'https://www.youtube.com/get_video_info?html5=1&c=TVHTML5&cver=6.20180913&el=detailpage&video_id=' + vid
+    r = util.urlopen(url)
+    if not r.ok:
+        raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+    else:
+        return r.text, None
+
+
+def youtube_get_new_endpoint(vid):
+    url = WATCH_ENDPOINT + vid
+    r = util.urlopen(url)
+    if not r.ok:
+        raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+
+    ipr = get_ipr(r.text)
+    if ipr is None:
+        try:
+            url = get_gdpr_consent_url(r.text)
+        except YouTubeError as e:
+            raise YouTubeError('Youtube "%s": No ytInitialPlayerResponse found and %s' % (url, str(e)))
+        r = util.urlopen(url)
+        if not r.ok:
+            raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+
+        ipr = get_ipr(r.text)
+        if ipr is None:
+            raise YouTubeError('Youtube "%s": No ytInitialPlayerResponse found' % url)
+
+    return None, ipr.group(1)
+
+
+def get_total_time(episode):
+    try:
+        vid = get_youtube_id(episode.url)
+        if vid is None:
+            return 0
+
+        url = WATCH_ENDPOINT + vid
+        r = util.urlopen(url)
+        if not r.ok:
+            return 0
+
+        ipr = get_ipr(r.text)
+        if ipr is None:
+            url = get_gdpr_consent_url(r.text)
+            r = util.urlopen(url)
+            if not r.ok:
+                return 0
+
+            ipr = get_ipr(r.text)
+            if ipr is None:
+                return 0
+
+        player_response = json.loads(ipr.group(1))
+        return int(player_response['videoDetails']['lengthSeconds'])  # 0 if live
+    except:
+        return 0
+
+
 def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
     if not preferred_fmt_ids:
         preferred_fmt_ids, _, _ = formats_dict[22]  # MP4 720p
@@ -197,27 +270,11 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
 
     vid = get_youtube_id(url)
     if vid is not None:
-        # TODO: changing 'detailpage' to 'embedded' allows age-restricted content
-        url = 'https://www.youtube.com/get_video_info?html5=1&el=detailpage&video_id=' + vid
-        r = requests.get(url)
-        if not r.ok:
-            logger.warning('Youtube get_video_info: %d %s' % (r.status_code, r.reason))
-
-            # TODO: watch URL does not work in europe due to GDPR cookie consent
-            url = 'https://www.youtube.com/watch?bpctr=9999999999&has_verified=1&v=' + vid
-            r = requests.get(url)
-            if not r.ok:
-                raise YouTubeError('%d %s' % (r.status_code, r.reason))
-
-            ipr = re.search(r'ytInitialPlayerResponse\s*=\s*({.+?})\s*;', r.text)
-            if ipr is None:
-                raise YouTubeError('No ytInitialPlayerResponse found')
-
-            old_page = None
-            new_page = ipr.group(1)
-        else:
-            old_page = r.text
-            new_page = None
+        try:
+            old_page, new_page = youtube_get_new_endpoint(vid)
+        except YouTubeError as e:
+            logger.info(str(e))
+            old_page, new_page = youtube_get_old_endpoint(vid)
 
         def find_urls(old_page, new_page):
             # streamingData is preferable to url_encoded_fmt_stream_map
@@ -242,7 +299,7 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
                     # playabilityStatus.liveStreamability.liveStreamabilityRenderer.displayEndscreen -- video has ended if present
 
                     if allow_partial and 'streamingData' in player_response and 'hlsManifestUrl' in player_response['streamingData']:
-                        r = requests.get(player_response['streamingData']['hlsManifestUrl'])
+                        r = util.urlopen(player_response['streamingData']['hlsManifestUrl'])
                         if not r.ok:
                             raise YouTubeError('HLS Manifest: %d %s' % (r.status_code, r.reason))
                         manifest = r.text.splitlines()
@@ -280,7 +337,7 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
         fmt_id_url_map = sorted(find_urls(old_page, new_page), reverse=True)
 
         if not fmt_id_url_map:
-            drm = re.search(r'(%22(cipher|signatureCipher)%22%3A|"signatureCipher"):', old_page or new_page)
+            drm = re.search(r'(%22(cipher|signatureCipher)%22%3A|"signatureCipher":)', old_page or new_page)
             if drm is not None:
                 raise YouTubeError('Unsupported DRM content')
             raise YouTubeError('No formats found')
@@ -289,8 +346,8 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
         fmt_id_url_map = dict(fmt_id_url_map)
 
         for id in preferred_fmt_ids:
-            if re.search(r'(^best|\+)', str(id)):
-                # skip formats that contain 'best.*' or a + (136+140)
+            if not re.search(r'^[0-9]+$', str(id)):
+                # skip non-integer formats 'best', '136+140' or twitch '720p'
                 continue
             id = int(id)
             if id in formats_available:
@@ -310,11 +367,8 @@ def get_real_download_url(url, allow_partial, preferred_fmt_ids=None):
     return url, duration
 
 
+@lru_cache(1)
 def get_youtube_id(url):
-    r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)\.swf', re.IGNORECASE).match(url)
-    if r is not None:
-        return r.group(1)
-
     r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/watch\?v=([^&]*)', re.IGNORECASE).match(url)
     if r is not None:
         return r.group(1)
@@ -323,6 +377,10 @@ def get_youtube_id(url):
     if r is not None:
         return r.group(1)
 
+    r = re.compile(r'http[s]?://(?:[a-z]+\.)?youtube\.com/v/(.*)\.swf', re.IGNORECASE).match(url)
+    if r is not None:
+        return r.group(1)
+
     return for_each_feed_pattern(lambda url, channel: channel, url, None)
 
 
@@ -346,6 +404,7 @@ def for_each_feed_pattern(func, url, fallback_result):
         r'http[s]?://(?:[a-z]+\.)?youtube\.com/profile?user=([a-z0-9]+)',
         r'http[s]?://(?:[a-z]+\.)?youtube\.com/rss/user/([a-z0-9]+)/videos\.rss',
         r'http[s]?://(?:[a-z]+\.)?youtube\.com/channel/([-_a-z0-9]+)',
+        r'http[s]?://(?:[a-z]+\.)?youtube\.com/feeds/videos.xml\?user=([a-z0-9]+)',
         r'http[s]?://(?:[a-z]+\.)?youtube\.com/feeds/videos.xml\?channel_id=([-_a-z0-9]+)',
         r'http[s]?://gdata.youtube.com/feeds/users/([^/]+)/uploads',
         r'http[s]?://gdata.youtube.com/feeds/base/users/([^/]+)/uploads',
@@ -370,26 +429,41 @@ def get_real_channel_url(url):
     return for_each_feed_pattern(return_user_feed, url, url)
 
 
-def get_channel_id_url(url):
+@lru_cache(1)
+def get_channel_id_url(url, feed_data=None):
     if 'youtube.com' in url:
         try:
-            req = util.urlopen(url)
+            if feed_data is None:
+                r = util.urlopen(url)
+                if not r.ok:
+                    raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+            else:
+                r = feed_data
             # video page may contain corrupt HTML/XML, search for tag to avoid exception
-            m = re.search(r'<meta itemprop="channelId" content="([^"]+)">', req.text)
+            m = re.search(r'<meta itemprop="channelId" content="([^"]+)">', r.text)
             if m:
                 channel_id = m.group(1)
             else:
-                raw_xml_data = io.BytesIO(req.content)
+                raw_xml_data = io.BytesIO(r.content)
                 xml_data = xml.etree.ElementTree.parse(raw_xml_data)
                 channel_id = xml_data.find("{http://www.youtube.com/xml/schemas/2015}channelId").text
+                if channel_id is None:
+                    # check entries if feed has an empty channelId
+                    m = re.search(r'<yt:channelId>([^<]+)</yt:channelId>', r.text)
+                    if m:
+                        channel_id = m.group(1)
+                    if channel_id is None:
+                        raise Exception('Could not retrieve YouTube channel ID for URL %s.' % url)
             channel_url = 'https://www.youtube.com/channel/{}'.format(channel_id)
             return channel_url
 
         except Exception:
-            logger.warning('Could not retrieve youtube channel id.', exc_info=True)
+            logger.warning('Could not retrieve YouTube channel ID for URL %s.' % url, exc_info=True)
+
+    raise Exception('Could not retrieve YouTube channel ID for URL %s.' % url)
 
 
-def get_cover(url):
+def get_cover(url, feed_data=None):
     if 'youtube.com' in url:
 
         class YouTubeHTMLCoverParser(HTMLParser):
@@ -414,8 +488,11 @@ def get_cover(url):
                     self.url.append(attribute_dict['src'])
 
         try:
-            channel_url = get_channel_id_url(url)
-            html_data = util.response_text(util.urlopen(channel_url))
+            channel_url = get_channel_id_url(url, feed_data)
+            r = util.urlopen(channel_url)
+            if not r.ok:
+                raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+            html_data = util.response_text(r)
             parser = YouTubeHTMLCoverParser()
             parser.feed(html_data)
             if parser.url:
@@ -426,7 +503,47 @@ def get_cover(url):
             logger.warning('Could not retrieve cover art', exc_info=True)
 
 
-def get_channel_desc(url):
+def get_gdpr_consent_url(html_data):
+    """
+    Creates the URL for automatically accepting GDPR consents
+    EU GDPR redirects to a form that needs to be posted to be redirected to a get request
+    with the form data as input to the youtube video URL. This extracts that form data from
+    the GDPR form and builds up the URL the posted form results.
+    """
+    class ConsentHTML(HTMLParser):
+        def __init__(self):
+            super().__init__()
+            self.url = ''
+            self.consentForm = False
+
+        def handle_starttag(self, tag, attributes):
+            attribute_dict = {attribute[0]: attribute[1] for attribute in attributes}
+            if tag == 'form' and attribute_dict['action'] == 'https://consent.youtube.com/s':
+                self.consentForm = True
+                self.url = 'https://consent.google.com/s?'
+            # Get GDPR form elements
+            if self.consentForm and tag == 'input' and attribute_dict['type'] == 'hidden':
+                self.url += '&' + attribute_dict['name'] + '=' + urllib.parse.quote_plus(attribute_dict['value'])
+
+        def handle_endtag(self, tag):
+            if tag == 'form':
+                self.consentForm = False
+
+    try:
+        parser = ConsentHTML()
+        parser.feed(html_data)
+    except Exception:
+        raise YouTubeError('Could not retrieve GDPR accepted consent URL')
+
+    if parser.url:
+        logger.debug('YouTube GDPR accept consent URL is: %s', parser.url)
+        return parser.url
+    else:
+        logger.debug('YouTube GDPR accepted consent URL could not be resolved.')
+        raise YouTubeError('No acceptable GDPR consent URL')
+
+
+def get_channel_desc(url, feed_data=None):
     if 'youtube.com' in url:
 
         class YouTubeHTMLDesc(HTMLParser):
@@ -445,8 +562,11 @@ def get_channel_desc(url):
                     self.description = attribute_dict['content']
 
         try:
-            channel_url = get_channel_id_url(url)
-            html_data = util.response_text(util.urlopen(channel_url))
+            channel_url = get_channel_id_url(url, feed_data)
+            r = util.urlopen(channel_url)
+            if not r.ok:
+                raise YouTubeError('Youtube "%s": %d %s' % (url, r.status_code, r.reason))
+            html_data = util.response_text(r)
             parser = YouTubeHTMLDesc()
             parser.feed(html_data)
             if parser.description:
@@ -486,6 +606,9 @@ def parse_youtube_url(url):
     logger.debug("Analyzing URL: {}".format(" ".join([scheme, netloc, path, query, fragment])))
 
     if 'youtube.com' in netloc:
+        if path == '/feeds/videos.xml' and re.search(r'^(user|channel|playlist)_id=.*', query):
+            return url
+
         if '/user/' in path or '/channel/' in path or 'list=' in query:
             logger.debug("Valid Youtube URL detected. Parsing...")
 
