@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # -*- coding: utf-8 -*-
 ####
 # 01/2011 Bernd Schlapsi <brot@gmx.info>
@@ -25,27 +24,33 @@
 
 import base64
 import datetime
+import logging
 import mimetypes
-import os
+
+from mutagen import File
+from mutagen.easyid3 import EasyID3
+from mutagen.easymp4 import EasyMP4Tags
+from mutagen.flac import Picture
+from mutagen.id3 import APIC, ID3
+from mutagen.mp3 import MP3, EasyMP3
+from mutagen.mp4 import MP4Cover, MP4Tags
 
 import gpodder
 from gpodder import coverart
 
-import logging
 logger = logging.getLogger(__name__)
 
-from mutagen import File
-from mutagen.flac import Picture
-from mutagen.mp3 import MP3
-from mutagen.id3 import ID3, APIC
-from mutagen.mp4 import MP4Cover
+
+# workaround for https://github.com/quodlibet/mutagen/issues/334
+# can't add_tags to MP4 when file has no tag
+MP4Tags._padding = 0
 
 _ = gpodder.gettext
 
 __title__ = _('Tag downloaded files using Mutagen')
 __description__ = _('Add episode and podcast titles to MP3/OGG tags')
 __authors__ = 'Bernd Schlapsi <brot@gmx.info>'
-__doc__ = 'http://wiki.gpodder.org/wiki/Extensions/Tagging'
+__doc__ = 'https://gpodder.github.io/docs/extensions/tagging.html'
 __payment__ = 'https://flattr.com/submit/auto?user_id=BerndSch&url=http://wiki.gpodder.org/wiki/Extensions/Tagging'
 __category__ = 'post-download'
 
@@ -56,14 +61,18 @@ DefaultConfig = {
     'always_remove_tags': False,
     'auto_embed_coverart': False,
     'set_artist_to_album': False,
+    'set_version': 4,
+    'modify_tags': True,
+    'remove_before_modify': False
 }
 
 
 class AudioFile(object):
-    def __init__(self, filename, album, title, genre, pubDate, cover):
+    def __init__(self, filename, album, title, subtitle, genre, pubDate, cover):
         self.filename = filename
         self.album = album
         self.title = title
+        self.subtitle = subtitle
         self.genre = genre
         self.pubDate = pubDate
         self.cover = cover
@@ -74,28 +83,46 @@ class AudioFile(object):
             audio.delete()
         audio.save()
 
-    def write_basic_tags(self):
+    def write_basic_tags(self, remove_before_modify, modify_tags, set_artist_to_album, set_version):
         audio = File(self.filename, easy=True)
 
+        if audio is None:
+            logger.warning("Unable to add tags to file '%s'", self.filename)
+            return
+
         if audio.tags is None:
             audio.add_tags()
 
-        if self.album is not None:
-            audio.tags['album'] = self.album
+        if modify_tags:
+            if remove_before_modify:
+                audio.delete()
 
-        if self.title is not None:
-            audio.tags['title'] = self.title
+            if self.album is not None:
+                audio.tags['album'] = self.album
 
-        if self.genre is not None:
-            audio.tags['genre'] = self.genre
+            if self.title is not None:
+                audio.tags['title'] = self.title
 
-        if self.pubDate is not None:
-            audio.tags['date'] = self.pubDate
+            if self.subtitle is not None:
+                audio.tags['subtitle'] = self.subtitle
 
-        if self.container.config.set_artist_to_album:
-            audio.tags['artist'] = self.album
+            if self.subtitle is not None:
+                audio.tags['comments'] = self.subtitle
 
-        audio.save()
+            if self.genre is not None:
+                audio.tags['genre'] = self.genre
+
+            if self.pubDate is not None:
+                audio.tags['date'] = self.pubDate
+
+            if set_artist_to_album:
+                audio.tags['artist'] = self.album
+
+        if type(audio) is EasyMP3:
+            audio.save(v2_version=set_version)
+        else:
+            # Not actually audio
+            audio.save()
 
     def insert_coverart(self):
         """ implement the cover art logic in the subclass
@@ -103,7 +130,7 @@ class AudioFile(object):
         None
 
     def get_cover_picture(self, cover):
-        """ Returns mutage Picture class for the cover image
+        """ Returns mutagen Picture class for the cover image
         Usefull for OGG and FLAC format
 
         Picture type = cover image
@@ -120,8 +147,8 @@ class AudioFile(object):
 
 
 class OggFile(AudioFile):
-    def __init__(self, filename, album, title, genre, pubDate, cover):
-        super(OggFile, self).__init__(filename, album, title, genre, pubDate, cover)
+    def __init__(self, filename, album, title, subtitle, genre, pubDate, cover):
+        super(OggFile, self).__init__(filename, album, title, subtitle, genre, pubDate, cover)
 
     def insert_coverart(self):
         audio = File(self.filename, easy=True)
@@ -131,8 +158,8 @@ class OggFile(AudioFile):
 
 
 class Mp4File(AudioFile):
-    def __init__(self, filename, album, title, genre, pubDate, cover):
-        super(Mp4File, self).__init__(filename, album, title, genre, pubDate, cover)
+    def __init__(self, filename, album, title, subtitle, genre, pubDate, cover):
+        super(Mp4File, self).__init__(filename, album, title, subtitle, genre, pubDate, cover)
 
     def insert_coverart(self):
         audio = File(self.filename)
@@ -143,13 +170,13 @@ class Mp4File(AudioFile):
             cover_format = MP4Cover.FORMAT_JPEG
 
         data = open(self.cover, 'rb').read()
-        audio.tags['covr'] =  [MP4Cover(data, cover_format)]
+        audio.tags['covr'] = [MP4Cover(data, cover_format)]
         audio.save()
 
 
 class Mp3File(AudioFile):
-    def __init__(self, filename, album, title, genre, pubDate, cover):
-        super(Mp3File, self).__init__(filename, album, title, genre, pubDate, cover)
+    def __init__(self, filename, album, title, subtitle, genre, pubDate, cover):
+        super(Mp3File, self).__init__(filename, album, title, subtitle, genre, pubDate, cover)
 
     def insert_coverart(self):
         audio = MP3(self.filename, ID3=ID3)
@@ -159,11 +186,11 @@ class Mp3File(AudioFile):
 
         audio.tags.add(
             APIC(
-                encoding = 3, # 3 is for utf-8
-                mime = mimetypes.guess_type(self.cover)[0],
-                type = 3,
-                desc = u'Cover',
-                data = open(self.cover).read()
+                encoding=3,  # 3 is for utf-8
+                mime=mimetypes.guess_type(self.cover)[0],
+                type=3,
+                desc='Cover',
+                data=open(self.cover, 'rb').read()
             )
         )
         audio.save()
@@ -172,6 +199,11 @@ class Mp3File(AudioFile):
 class gPodderExtension:
     def __init__(self, container):
         self.container = container
+        # fix #737 EasyID3 doesn't recognize subtitle and comment tags
+        EasyID3.RegisterTextKey("comments", "COMM")
+        EasyID3.RegisterTextKey("subtitle", "TIT3")
+        EasyMP4Tags.RegisterTextKey("comments", "desc")
+        EasyMP4Tags.RegisterFreeformKey("subtitle", "SUBTITLE")
 
     def on_episode_downloaded(self, episode):
         info = self.read_episode_info(episode)
@@ -183,32 +215,29 @@ class gPodderExtension:
     def get_audio(self, info, episode):
         audio = None
         cover = None
+        audioClass = None
 
         if self.container.config.auto_embed_coverart:
             cover = self.get_cover(episode.channel)
 
         if info['filename'].endswith('.mp3'):
-            audio = Mp3File(info['filename'],
-                info['album'],
-                info['title'],
-                info['genre'],
-                info['pubDate'],
-                cover)
+            audioClass = Mp3File
         elif info['filename'].endswith('.ogg'):
-            audio = OggFile(info['filename'],
-                info['album'],
-                info['title'],
-                info['genre'],
-                info['pubDate'],
-                cover)
+            audioClass = OggFile
         elif info['filename'].endswith('.m4a') or info['filename'].endswith('.mp4'):
-            audio = Mp4File(info['filename'],
+            audioClass = Mp4File
+        elif File(info['filename'], easy=True):
+            # mutagen can work with it: at least add basic tags
+            audioClass = AudioFile
+
+        if audioClass:
+            audio = audioClass(info['filename'],
                 info['album'],
                 info['title'],
+                info['subtitle'],
                 info['genre'],
                 info['pubDate'],
                 cover)
-
         return audio
 
     def read_episode_info(self, episode):
@@ -216,6 +245,7 @@ class gPodderExtension:
             'filename': None,
             'album': None,
             'title': None,
+            'subtitle': None,
             'genre': None,
             'pubDate': None
         }
@@ -233,6 +263,8 @@ class gPodderExtension:
         else:
             info['title'] = title
 
+        info['subtitle'] = episode._text_description
+
         if self.container.config.genre_tag is not None:
             info['genre'] = self.container.config.genre_tag
 
@@ -256,12 +288,15 @@ class gPodderExtension:
         if self.container.config.always_remove_tags:
             audio.remove_tags()
         else:
-            audio.write_basic_tags()
+            audio.write_basic_tags(self.container.config.remove_before_modify,
+                                   self.container.config.modify_tags,
+                                   self.container.config.set_artist_to_album,
+                                   self.container.config.set_version)
 
             if self.container.config.auto_embed_coverart:
                 audio.insert_coverart()
 
-        logger.info(u'tagging.on_episode_downloaded(%s/%s)', episode.channel.title, episode.title)
+        logger.info('tagging.on_episode_downloaded(%s/%s)', episode.channel.title, episode.title)
 
     def get_cover(self, podcast):
         downloader = coverart.CoverDownloader()
